pro dopmap,infile=infile,intfac=fac,velscale=vfac,phasesize=phf,velsize=vf,size=imf, $
  reco=reco,diff=diff,read=re,overs=overs,eps=eps,pscolor=pscolor
; recompute dopmap if input data changed, plot dopplermap. 

;infile: pathname of output file from dop.f to be plotted. Default: 'dop.out'
;fac:    contrast factor for greyscale plots of spectrum (needed only for ps output,
;   on screen contrast is better manipulated with XLOADCT). Default: fac=1
;vfac:   unit of velocity for the axes. Default: 1 cm/s
;phf:    size of screen image of spectrum in phase direction. Default phf=1 gives 300 pixels.
;vf:     size of screen image of spectrum in velocity direction. Default phf=1 gives 170 pixels.
;imf:    size of screen image of dopmapm. Default phf=1 gives 200 pixels.
;reco:   reco=1 if reconstructed spectrum to be shown on screen. Default reco=1.
;diff:   diff=1 if O-C, C-O difference spectra to be shown. Default: diff=1.
;re:     force re-reading of infile (for debug purpose, should be handled automatically).
;overs:  oversampling factor of doppler map image. Default: overs=1
;eps:    eps=1 gives encapsulated postscript output. Default: eps=0, gives printable output
;pscolor: color postscript. Default: no color postscript

common mapdata,nph,nvp,nv,w0,gamma,pha,vp,dm,ih,iw,pb0,pb1,ns,ac,al,clim,norm,wid,af,$
  va,im,dmr,dpx,abs,atm,cfi
common savemappar, infs,facs,vfacs,phfs,vfs,imfs,recos,diffs,overss,fidents,epss,pscs

;check keywords
if n_elements(infile) eq 0 then if n_elements(infs) eq 0 then infile='dop.out' else infile=infs
infs=infile
if n_elements(fac) eq 0 then if n_elements(facs) eq 0 then fac=1 else fac=facs
facs=fac
if n_elements(vfac) eq 0 then if n_elements(vfacs) eq 0 then vfac=1e8 else vfac=vfacs
vfacs=vfac
if n_elements(phf) eq 0 then if n_elements(phfs) eq 0 then phf=1 else phf=phfs
phfs=phf
if n_elements(vf) eq 0 then if n_elements(vfs) eq 0 then vf=1 else vf=vfs
vfs=vf
if n_elements(imf) eq 0 then if n_elements(imfs) eq 0 then imf=1 else imf=imfs
imfs=imf
if n_elements(reco) eq 0 then if n_elements(recos) eq 0 then reco=1 else reco=recos
if reco gt 0 then reco=1 else reco=0
recos=reco
if n_elements(diff) eq 0 then if n_elements(diffs) eq 0 then diff=1 else diff=diffs
if diff gt 0 then diff=1 else diff=0
diffs=diff
if n_elements(overs) eq 0 then if n_elements(overss) eq 0 then overs=1.0 else overs=overss
overss=overs
if n_elements(eps) eq 0 then if n_elements(epss) eq 0 then eps=0 else eps=epss
epss=eps
if n_elements(pscolor) eq 0 then if n_elements(pscs) eq 0 then pscolor=0 else pscolor=pscs
pscs=pscolor

; if 'dop.out' to be plotted, synchronize it, recompiling/rerunning 'dop' if needed
if strtrim(infile,2) eq 'dop.out' then syncdop,ov=overs

; peek at infile to see if it's new, if yes, read it.
get_lun,lun
openr,lun,infile
fident=1d0
readf,lun, dum,dum,dum,dum,fident
close,lun
oldid=0
if n_elements(fidents) ne 0 then if fidents eq fident then oldid=1 
if oldid eq 0 or n_elements(re) eq 1 then reamap,infile
fidents=fident

ntab=3                         ; color table red-temperature
set_plot,'ps'
if pscolor eq 1 then begin
  device,/color
  loadct,ntab,/silent
endif else device,color=0
device,/landscape,font_size=12,bits_per_pixel=8,xsize=30,ysize=30
device,filename='dopmap.ps'
if eps eq 1 then device,/encaps else device,encaps=0
del=0.04
xsiz=0.15
ysiz=0.3
vsiz=0.22
mcolp=!d.table_size-1 ; max no of colors of display
set_plot,'x'
loadct,ntab,/silent
off =0
vpi=vp/vfac
dm1=dm
; set screen window for the spectra
xsi=170*phf
ysi=300*vf
spac=5
if reco eq 1 then xpix=xsi+spac else xpix=0
if diff eq 1 then ypix=ysi+spac else ypix=0
window,0,xsiz=xsi+xpix,ysiz=ysi+ypix
mcol=!d.table_size-1 ; max no of colors of display
if mcol lt 63 then print, 'display appears to have only',mcol,'colors'
; replicate spectrum in phase direction
d=fltarr(2*nph,nvp)
for i=0,nph-1 do begin
  d(i,*)=dm(i,*)
  d(i+nph,*)=dm(i,*)
endfor

; plot input spectrum
dma=max(dm)
d=d/dma*fac
; x-window plot
wset,0
tv,congrid(rotate((mcol*(1-d)>0)<mcol,4),xsi,ysi),0.,0.
; ps plot
set_plot,'ps'
loadct,ntab,/silent
plot,vpi,pha,position=[0,off,xsiz,ysiz+off],xsty=5,ysty=5, $
  /nodata
yl=!y.window(1)-!y.window(0)
xl=!x.window(1)-!x.window(0)
tv,rotate((mcolp*(1-d)>0)<mcolp,4),!x.window(0),!y.window(0),$
  xsize=xl,ysize=yl,/norm
!y.range=[0,2]
plot,vpi,pha,position=[0,off,xsiz,ysiz+off],psym=3,xsty=1,ysty=1,$
  xtit='!5v',ytit='phase',/noerase,/nodat

;string for top of ps page
lab=string(format='(''!5ih='',i1,'' iw='',i1,'' pb0,1:'',2f6.3)',ih,iw,pb0,pb1)
lab=lab+string(format='('' ns='',i3,'' !7a!5='',f8.5,'' norm='',i1)',ns,al,norm)
if abs then lab=lab+'  ABSORPTION  '
lab=lab+string(format='('' nph,nvp,nv:'',3i4)',nph,nvp,nv)
lab=lab+string(format='('' !7k!5!D0!N='',f9.2,'' !7c!5='',e10.2)',w0,gamma)
lab=lab+string(format='('' wid='',e9.1,'' af='',f7.2)',wid,af)+'   '+cfi
xyouts,0,0.6*!d.y_size,lab,charsize=0.7,/device

; velocity coordinates of centers of bins
vxy=(2*findgen(nv)/(nv-1)-1)*va/vfac
dv=vxy(1)-vxy(0)
vxy=[vxy(0)-dv/2,vxy+dv/2]

; stretch dopmap to fixed dimension
nm1=200*imf
im1=congrid(im,nm1,nm1)
am=max(im1)
sc=2
; first scaling: linear
im2=im1/am>0
!y.range=[min(vxy),max(vxy)]
set_plot,'x'
  window,2,xsi=2*nm1+spac,ysi=2*nm1+spac
  tv,mcol*(1-im2)>0
set_plot,'ps'
  pos=[0,off+ysiz+del,vsiz,ysiz+off+del+vsiz]
  plot,vxy,vxy,position=pos,xsty=5,ysty=5,/nodata, $
  /noerase
  yl=!y.window(1)-!y.window(0)
  xl=!x.window(1)-!x.window(0)
  tv,(mcolp*(1-im2)>0)<mcolp,!x.window(0),!y.window(0),$
    xsize=xl,ysize=yl,/norm
  plot,vxy,vxy,position=pos,psym=3,xsty=1,ysty=1,$
    xtit='!5v!Dx!N',ytit='v!Dy!N',/noerase,/nodat
  fi=2*!pi*findgen(101)/100.

sc=2
; second scaling :log
im2=(alog((im1/am)>0.0001)/sc+1) >0
set_plot,'x'
  wset,2
  tv,mcol*(1-im2),0,nm1+spac
set_plot,'ps'
  pos=pos+[vsiz+del,0,vsiz+del,0]
  plot,vxy,vxy,position=pos,xsty=5,ysty=5,/nodata, $
  /noerase
  yl=!y.window(1)-!y.window(0)
  xl=!x.window(1)-!x.window(0)
  tv,(mcolp*(1-im2)>0)<mcolp,!x.window(0),!y.window(0),$
    xsize=xl,ysize=yl,/norm
  plot,vxy,vxy,position=pos,psym=3,xsty=1,ysty=1,$
   /noerase,/nodat
  fi=2*!pi*findgen(100)/100.

; third: log over larger range
sc=4
im2=(alog((im1/am)>0.0001)/sc+1)>0
set_plot,'x'
  wset,2
  tv,mcol*(1-im2),nm1+spac,nm1+spac
set_plot,'ps'
  pos=pos+[vsiz+del,0,vsiz+del,0]
  plot,vxy,vxy,position=pos,xsty=5,ysty=5,/nodata, $
  /noerase
  yl=!y.window(1)-!y.window(0)
  xl=!x.window(1)-!x.window(0)
  tv,(mcolp*(1-im2)>0)<mcolp,!x.window(0),!y.window(0),$
    xsize=xl,ysize=yl,/norm
  plot,vxy,vxy,position=pos,psym=3,xsty=1,ysty=1,$
    /noerase,/nodat
  fi=2*!pi*findgen(100)/100.

; difficult pixel plot
for j=0,3 do begin
  im1=dpx(*,*,j)
  im1=im1/max(im1)
  im1=mcol*(0.5+2*im1<1>0)
  set_plot,'x'
  wset,2
  dx=nm1/2*((j eq 1) or (j eq 3))
  dy=nm1/2*((j eq 0) or (j eq 1))
  tv,congrid(im1,nm1/2-1,nm1/2-1),nm1+spac+dx,dy
endfor

;replicate reconstructed spectrum in phase direction
dmx=max(dmr)
d1=fltarr(2*nph,nvp)
for i=0,nph-1 do begin
  d1(i,*)=dmr(i,*)
  d1(i+nph,*)=dmr(i,*)
endfor

d1=d1/dma*fac
set_plot,'x'
wset,0
if reco eq 1 then tv,congrid(rotate(mcol*(1-d1<mcol)>0,4),xsi,ysi),xsi+spac,0
if reco eq 1 then begin
  set_plot,'ps'
  pos=[xsiz+del,off,2*xsiz+del,ysiz+off]
  plot,vpi,pha,position=pos,xsty=5,ysty=5,/nodata,yra=[0,2], $
  xtit='!5v',ytit='phase',/noerase
  yl=!y.window(1)-!y.window(0)
  xl=!x.window(1)-!x.window(0)
  tv,rotate((mcolp*(1-d1)>0)<mcolp,4),!x.window(0),!y.window(0),$
    xsize=xl,ysize=yl,/norm
  !y.range=[0,2]
  plot,vpi,pha,position=pos,psym=3,xsty=1,ysty=1,$
    /noerase,/nodat
endif
set_plot,'x'
d2=congrid(rotate(d1-d,4),xsi,ysi)
if diff eq 1 then tv,((d2+0.5)*mcol<mcol)>0,0,ysi+spac
if diff eq 1 then tv,((-d2+0.5)*mcol<mcol)>0,xsi+spac,ysi+spac
if diff eq 1 then begin
  set_plot,'ps'
  pos=[2*(xsiz+del),off,2*(xsiz+del)+xsiz,ysiz+off]
  plot,vpi,pha,position=pos,xsty=5,ysty=5,/nodata, $
  xtit='!5v',ytit='phase',/noerase
  yl=!y.window(1)-!y.window(0)
  xl=!x.window(1)-!x.window(0)
  tv,rotate(((fac*(d1-d)+0.5)*mcolp>0)<mcolp,4),!x.window(0),!y.window(0),$
    xsize=xl,ysize=yl,/norm
  !y.range=[0,2]
  plot,vpi,pha,position=pos,psym=3,xsty=1,ysty=1,$
    /noerase,/nodat
  pos=[3*(xsiz+del),off,3*(xsiz+del)+xsiz,ysiz+off]
  plot,vpi,pha,position=pos,xsty=5,ysty=5,/nodata, $
  xtit='!5v',ytit='phase',/noerase
  yl=!y.window(1)-!y.window(0)
  xl=!x.window(1)-!x.window(0)
  tv,rotate(((fac*(d-d1)+0.5)*mcolp>0)<mcolp,4),!x.window(0),!y.window(0),$
    xsize=xl,ysize=yl,/norm
  !y.range=[0,2]
  plot,vpi,pha,position=pos,psym=3,xsty=1,ysty=1,$
    /noerase,/nodat
  device,/close_file
  set_plot,'x'
endif

!y.range=0 ;reset to default
device ;reset to default
free_lun,lun
return
end
