pro foldspec,fcol,tcol,dcol,wcol,scol,cfile=cfi,nbin=nbin,dir=dir,dpha=dpha
; take spectra from a list and fold in phase, rebinning equidistantly in phase
; the wavelength scale is take from the first file, the next files are
; interpolated onto this scale.

; fcol: column number of file name in list of spectra
; tcol: column number of phase value
; dcol: column number of phase interval
; wcol: column number of wavelength in spectrum file
; scol: column number of spectrum
; cfile: name of file containing list of spectra. Default: 'specnames'
; nbin:  number of phase bins to be used. Default: 1.5*(number of spectra)
; dir: prefix to be added to file names from list. Default: no prefix
; dph: a fixed phase interval can be specified here for the exposure times.
common foldsavepar,fcols,tcols,dcols,wcols,scols,cfis,nbins,dirs,dphas
if n_elements(cfi) eq 0 then if n_elements(cfis) ne 0 then cfi=cfis else begin
  cfi='specnames'
  ex=findfile(cfi)
  ex=ex(0)
  if ex eq '' then begin
    print,'file name of list of spectra?'
    read,cfi
    cfi=strtrim(cfi,2)
  endif
endelse
ex=findfile(cfi)
ex=ex(0)
if ex eq '' then begin
  print,cfi+' (list of input spectra) not found'
  stop
endif
cfis=cfi
if n_elements(fcol) eq 0 then if n_elements(fcols) ne 0 then fcol=fcols else begin
  fcol=0
  print,'which column number in '+cfi+' is file name?'
  read,fcol
endelse
fcols=fcol
if n_elements(tcol) eq 0 then if n_elements(tcols) ne 0 then tcol=tcols else begin
  tcol=0
  print,'which column number is phase?'
  read,tcol
endelse
tcols=tcol
if n_elements(dcol) eq 0 then if n_elements(dcols) ne 0 then dcol=dcols else begin
  dcol=0
  print,'which column number is phase interval? (reply 0 if not present)'
  read,dcol
endelse
dcols=dcol
if n_elements(wcol) eq 0 then if n_elements(wcols) ne 0 then wcol=wcols else begin
  wcol=0
  print,'which column number in spectrum file is wavelength?'
  read,wcol
endelse
wcols=wcol
if n_elements(scol) eq 0 then if n_elements(scols) ne 0 then scol=scols else begin
  scol=0
  print,'which column number in spectrum file is spectrum?'
  read,scol
endelse
scols=scol
if n_elements(dir) eq 0 then if n_elements(dirs) eq 0 then dir='' else dir=dirs
dir=strtrim(dir,2)
dirs=dir
spawn,'wc '+cfi,wco     ; count no of lines of file list
nfile=0
reads,wco,nfile

get_lun,lun
get_lun,lun1
openr,lun,cfi
nfi=0
fnam=strarr(nfile)
pha=fltarr(nfile)
dphase=pha
dphav=0
for il=0,nfile-1 do begin 
  ist=''
  readf,lun,ist               ; input a line of txt
  ist=strtrim(strcompress(ist),2)
  if ist ne '' then begin     ; if line not empty, parse and read
    parts=str_sep(ist,' ')    ; separate into fields
    nam=parts(fcol-1)         ; get file name
    if il eq 0 then nam1=nam
    reads,parts(tcol-1),ph    ; get phase
    if dcol eq 0 then begin
      if n_elements(dpha) ne 0 then dph=dpha else dph=1./nfile 
    endif else reads,parts(dcol-1),dph          ; take dph from file
    dphav=dphav+dph
    dphase(nfi)=dph
; fold phase between (0,1)
    pha(nfi)=ph
    if ph lt 0. then pha(nfi)=ph-fix(ph)+1    
    if ph gt 1. then pha(nfi)=ph-fix(ph)
    if dir ne '' then nam='/'+nam
    fnam(nfi)=dir+nam                  ; add prefix for file names
    nfi=nfi+1
  endif
endfor
close,lun
dphav=dphav/nfile
nfile=n_elements(pha)
if n_elements(nbin) eq 0 then begin 
  if n_elements(nbins) eq 0 then begin 
    if dcol eq 0 then nbin=nfile else nbin=fix(1.5/dphav)
  endif else nbin=nbins
endif
nbins=nbin
bin=findgen(nbin+1)/nbin
bin=[bin(0:nbin-1)-1,bin]  ; bins now from phase -1 to 1
wt=fltarr(2*nbin)

for il=0,nfile-1 do begin
  name=fnam(il)
  print,name,pha(il)
  spawn,'wc '+name,wco
  nwo=0
  reads,wco,nwo                 ; read no of lines
  if il eq 0 then begin         ; define spectrum and wavelength arrays
    sp=fltarr(nwo)
    w=sp
    trsp=fltarr(2*nbin,nwo)
    strin=strarr(nwo)
  endif
  
  openr,lun1,name                ; open spectrum file
  readf,lun1,strin
  nc=n_elements(str_sep(strtrim(strcompress(strin(0)),2),' '))  ; no of columns
  strdat=strarr(nwo,nc)
  ni=0
  for i=0,nwo-1 do begin
    strdat(i,*)=str_sep(strtrim(strcompress(strin(i)),2),' ')
    if n_elements(strdat(i,*)) gt 1 then begin  ; skip if line has less than 2 columns
      x=0.
      reads,strdat(i,wcol-1),x
      w(ni)=x  
      reads,strdat(i,scol-1),x
      sp(ni)=x
      ni=ni+1
    endif
  endfor
  close,lun1
  nwo=ni 
  if il eq 0 then nwo1=nwo else if nwo1 ne nwo then begin
    print,'file '+name+'has different length from '+nam1
    stop
  endif
; interpolate onto wavelength scale of first file
  if il eq 0 then w1=w else sp=interpol(sp,w,w1) 

; determine how much of the spectrum goes into each bin
  ph=pha(il)
  ph0=ph-dph/2
  ph1=ph+dph/2
  for ib=0,2*nbin-1 do begin
    r=bin(ib+1)
    l=bin(ib)
    if ph0 lt r and ph1 gt l then begin
      wph=min([r,ph1])-max([l,ph0])
      wt(ib)=wt(ib)+wph
      trsp(ib,*)=trsp(ib,*)+wph*sp
    endif
  endfor
endfor
close,lun
wt(nbin:2*nbin-1)=wt(nbin:2*nbin-1)+wt(0:nbin-1)
wt=wt(nbin:2*nbin-1)
trsp(nbin:2*nbin-1,*)=trsp(nbin:2*nbin-1,*)+trsp(0:nbin-1,*)
trsp=trsp(nbin:2*nbin-1,0:nwo-1)
wt=wt/total(wt)*nbin     ;normalize weights by their avg
print,wt
; for bins with weight <1 add a bit of neighboring bins
; find first bin with wt>1
i=0 
while wt(i) lt 1 do i=i+1
i0=i
tup=trsp
wup=wt
tdn=trsp
wdn=wt
ssw=sqrt(sqrt(wt))
; upward
for i=i0+1,i0+nbin-1 do begin 
  im=i mod nbin
  imm=(i-1) mod nbin
  if wup(im) lt 1 then begin
    if wup(im) gt 0 then tup(im,*)=ssw(im)*tup(im,*)/wup(im) else tup(im,*)=0
    tup(im,*)=tup(im,*)+(1-ssw(im))*tup(imm,*)/wup(imm)
    wup(im)=0.5*wup(imm)
    tup(im,*)=wup(im)*tup(im,*)
  endif
endfor
; downward
for i=i0-1,i0-nbin+1,-1 do begin 
  im=(i+nbin) mod nbin
  imm=(i+1+nbin) mod nbin
  if wdn(im) lt 1 then begin
    if wdn(im) gt 0 then tdn(im,*)=ssw(im)*tdn(im,*)/wdn(im) else tdn(im,*)=0
    tdn(im,*)=tdn(im,*)+(1-ssw(im))*tdn(imm,*)/wdn(imm)
    wdn(im)=0.5*wdn(imm)
    tdn(im,*)=wdn(im)*tdn(im,*)
  endif
endfor
for ib=0,nbin-1 do trsp(ib,*)=(tup(ib,*)+tdn(ib,*))/(wup(ib)+wdn(ib))

openw,lun,'foldspec'
writeu,lun,fix(nbin),fix(nwo)
writeu,lun,w(0:nwo-1)
writeu,lun,trsp
writeu,lun,wt
printf,lun,cfi
close,lun
free_lun,lun1
free_lun,lun
plfold,1.5
return
end

