pro k1,vmax=vma,spotmask=phic,eps=eps
; determine K1 from doppler map by measuring offsets in velocity rings. 
; Output is a plot. 6 different 
; binnings done to get better impression of error in the result. NOTE: since
; these binnings are done on the same data, the points in plot are not independent.
;
; vmax: max velocity of rings to use. Default: max velocity of map
; spotmask: 2-element array containing sector to exclude for the measurement, in 
; degrees counterclockwise from +v_x axis in doppler map. Default: no excluded 
; sector. example: the call k1,sp=[90,180] will ignore second quandrant of map. NOTE:
; these angles are defined the math way, and differ from the def of phase angle of K1.
; The output shows amplitude and phase of K1 velocity, where phase is measured
; relative to expected radial velocity curve of M1 (negative value means that zero
; crossings are delayed w/r expectation)

get_lun,lun
openr,lun,'dop.out'
readf,lun,nph,nvp
pha=fltarr(nph)
vp=fltarr(nvp)
readf,lun,pha
readf,lun,vp
dm=fltarr(nph,nvp)
readf,lun,dm
readf,lun,dum
readf,lun,nv,va
vxy=(2*findgen(nv)/(nv-1)-1)*va
dv=vxy(1)-vxy(0)
im=fltarr(nv,nv)
readf,lun,im
close,lun

if n_elements(phic) eq 0 then phic=[0.,0.]
ph0=phic(0)*!pi/180 ; sector boundaries of igored area (hot spot)
ph1=phic(1)*!pi/180
s=0.8*va/nv ; stepsize in minimum search
fa=[1.2,1.2,1.3,1.3,1.4,1.4] ; different binnings for statistics
na=[3,4,3,4,3,4]
for ifa=0,n_elements(fa)-1 do begin
  n0=na(ifa)
  fac=fa(ifa) 
  vmin=n0*dv
  if n_elements(vma) ne 0 then vam=min([va,vma]) else vam=va
  nnv=fix(alog(vam/vmin)/alog(fac))
  v1a=vmin*exp(alog(fac)*indgen(nnv+1))
  v00=fltarr(nnv)
  k1xa=v00
  k1ya=v00
  k1x=0
  k1y=0
  kmax=va/5
  itm=30
  for iv=0,nnv-1 do begin
    v0=v1a(iv)
    v1=v1a(iv+1)
    v00(iv)=(v1+v0)/2
    is=0
    ct=(1 gt 0)
    it=0
    while is eq 0 and it lt itm and ct do begin
      it=it+1
      rr0=rmr(k1x,k1y,v0,v1,ph0,ph1,im,vxy,nv,msk,vax,vax2,vay,vay2)
      rr1=rmr(k1x+s,k1y,v0,v1,ph0,ph1,im,vxy,nv,msk,vax,vax2,vay,vay2)
      rr2=rmr(k1x,k1y+s,v0,v1,ph0,ph1,im,vxy,nv,msk,vax,vax2,vay,vay2)
      rr3=rmr(k1x-s,k1y,v0,v1,ph0,ph1,im,vxy,nv,msk,vax,vax2,vay,vay2)
      rr4=rmr(k1x,k1y-s,v0,v1,ph0,ph1,im,vxy,nv,msk,vax,vax2,vay,vay2)
        if rr0 lt rr1 and rr0 lt rr2 and rr0 lt rr3 and rr0 lt rr4 then begin
        k1xa(iv)=k1x+fit(rr0,rr1,rr3,s)
        k1ya(iv)=k1y+fit(rr0,rr2,rr4,s)
        is=1
      endif else begin
        de=abs(rr1-rr3)+abs(rr2-rr4)
        if de eq 0. then de=1e-10
        dx=(rr3-rr1)/de*0.5*s
        dy=(rr4-rr2)/de*0.5*s
        k1x=k1x+dx
        k1y=k1y+dy
        ct=(k1x lt kmax and k1y lt kmax)
      endelse
    endwhile
    print,'v,k1x,k1y ',string(format='(3e10.2,i3)',v00(iv),k1xa(iv),k1ya(iv),is)
    if is eq 0 then begin
      k1x=0
      k1y=0
    endif
  endfor
  k1ya= k1ya+(k1xa eq 0.)*1e-5*va ;prevent 0/0 divisions
  k1a=sqrt(k1xa^2+k1ya^2)
  phi=atan(-k1xa/k1ya)*180/!pi    ; K1 phase, 
  phi=phi+(k1ya gt 0.)*!pi        ; and get them into a conventional interval
  if ifa eq 0 then begin          ; first time round define arrays
    v0t=v00
    k1t=k1a
    pht=phi
  endif else begin 
    v0t=[v0t,v00]
    k1t=[k1t,k1a]
    pht=[pht,phi]
  endelse
endfor

; bin points for plot of average values
in=sort(v0t)
v0t=v0t(in)
k1t=k1t(in)
pht=pht(in)
i=0
iv=n_elements(in)-1
fb=5
nbi=(iv-1)/fb+1
k1b=fltarr(nbi)
phb=k1b
v0b=k1b
for ib=0,nbi-1 do begin
  i0=ib*fb
  i1=min([i0+fb,iv])
  v0b(ib)=total(v0t(i0:i1-1))/(i1-i0)
  k1b(ib)=total(k1t(i0:i1-1))/(i1-i0)
  phb(ib)=total(pht(i0:i1-1))/(i1-i0)
endfor

vsca=1e5
!p.multi=[0,1,2]
scstr=' ('+string(format='(e7.0)',vsca)+' cm/s)'
tst='spotmask='+string(format='(i4,'','',i4)',fix(phic))
set_plot,'x'
plot,v0t/vsca,k1t/vsca,xtit='v'+scstr,title=tst, $
  ytit='k1'+scstr,psym=2,xra=[0,vam/vsca],xsty=1
oplot,v0b/vsca,k1b/vsca
plot,v0t/vsca,pht,psym=1,xra=[0,vam/vsca],xsty=1, $
  xtit='v'+scstr,ytit='!7u!5 (degrees)'
oplot,v0b/vsca,phb

set_plot,'ps'
device,filename='k1.ps',/portrait,xsize=18,ysize=25,xoff=2,yoff=2
if keyword_set(eps) then device,/encaps
plot,v0t/vsca,k1t/vsca,xtit='v'+scstr,title=tst, $
  ytit='k1'+scstr,psym=2,xra=[0,vam/vsca],xsty=1
oplot,v0b/vsca,k1b/vsca
plot,v0t/vsca,pht,psym=1,xra=[0,vam/vsca],xsty=1, $
  xtit='v'+scstr,ytit='!7u!5 (degrees)'
oplot,v0b/vsca,phb
device,/close_file
set_plot,'x'
!p.multi=[0,1,1]
free_lun,lun
return
end

